## Griffiths et al 2021, X-ray CT reveals 4D root system development and lateral root responses to nitrate in soil
#dataprocessing

###### MCT1 candiates experiment
library(tidyverse)      #includes packages ggplot2, dplyr, tidyr, readr, purrr, tibble, stringr, forcats
library(patchwork)    #for combining figures into a single plot
library(GGally)       #pairwise graphs
library(broom)
library(lmerTest) #lme4, Matrix, lmer, step

dat <- read_csv(file=paste("MCT1_alldata_2021.csv", sep=""), na = c("NA", "na", "n.a.", ""))
NUpdata <- read_csv(file=paste("MCT1_simrootdata_2021_long.csv", sep=""), na = c("NA", "na", "n.a.", ""))
fulldata <- left_join(dat, NUpdata)

names(fulldata)
head(fulldata)
summary(fulldata) #summary of your data, to sumarise by column write "summary(fulldata$COLUMN_NAME)"
unique(fulldata$Treatment) #check no errors in factor (Treatment) name

is.factor(fulldata$Geno)
fulldata$Geno <- as.factor(fulldata$Geno)

is.factor(fulldata$Treatment)
fulldata$Treatment <- as.factor(fulldata$Treatment)

is.factor(fulldata$Rep)
fulldata$Rep <- as.factor(fulldata$Rep)

########################
## Calculate measures ##
########################
fulldata <- fulldata %>% 
  mutate(MLS_mm = (TLS_mm / SC)
         ,MLL_mm = (TLL_mm / LC)
         ,SRL_m.g = (TLA_mm/1000)/(DRW_mg*1000)
         )

####################################
## Define standard error function ##
####################################
stderror <- function(x) sd(na.omit(x))/sqrt(length(na.omit(x)))   #define standard error function 

#################################################
## Remove unnecessary columns for mean groupings ##
#################################################
dat <- fulldata %>% select(-matches(c("Rep"))) #keep geno/treatment/variables, remove replicate number

##########################
## Process means/errors ##
##########################
names(dat) #find column numbers for variables only eg.columns2:17
names(dat[3:51])
varcol <- 3:51

dat_mean <- dat %>% group_by(Geno
                             ,Treatment
                             ) %>% summarize_at(colnames(.)[varcol], mean, na.rm = TRUE)


dat_se <- dat %>% group_by(Geno
                           ,Treatment
                           ) %>% summarize_at(colnames(.)[varcol], stderror)

####################
## Add mean & error column names
####################
paste(names(dat_mean)[varcol],"_mean", sep = "")
paste(names(dat_se)[varcol],"_se", sep = "")
names(dat_mean)[varcol] <- paste(names(dat_mean)[varcol],"_mean", sep = "")
names(dat_se)[varcol] <- paste(names(dat_se)[varcol],"_se", sep = "")

####################
## Merge means & errors ##
####################
dat_meanErrors <- merge(dat_mean, dat_se, by=c("Geno"
                                               ,"Treatment"
                                               ))
names(dat_meanErrors)

###########################
## Export meanErrors csv ##
###########################
dir.create("output",showWarnings = TRUE)
write_csv(dat_meanErrors, file="output/MCT1_MeanErrors.csv")


##########################################
## Define function for ANOVA & TukeyHSD ##
##            Treatment * Day           ##
##########################################
dir.create("output/",showWarnings = TRUE)
dir.create("output/stat_analysis",showWarnings = TRUE)
dir.create("output/stat_analysis/TukeyHSD",showWarnings = TRUE)

make_significance_stars <- function(pval) {
  stars = ""
  if(pval <= 0.001)
    stars = "***"
  if(pval > 0.001 & pval <= 0.01)
    stars = "**"
  if(pval > 0.01 & pval <= 0.05)
    stars = "*"
  if(pval > 0.05)
    stars = "ns"
  stars
}

## define anova function
compute_aov <- function(x)
{
  dat <- x
  dat[[1]] <- factor(dat[[1]])
  dat[[2]] <- factor(dat[[2]])
  
  result_output <- data.frame()
  
  for (i in 3:length(colnames(dat)))
  {
    print(colnames(dat)[i])
    # if(i != 32 && i != 33 && i != 45 && i != 46) #drop variables which are not present in both treatments
    {
      anova_output <- aov(dat[[i]] ~ Geno * Treatment, data = dat)
      anova_output <- broom::tidy(anova_output)
      anova_output <- anova_output %>%
        mutate(trait = colnames(dat)[i])
      # %>%
      # mutate(signif = sapply(p.value, function(x) make_significance_stars(x))) %>%
      # mutate(statistic_2dp = formatC(statistic, digits=2,format="f")) %>%
      # mutate(fvalue_significance = paste(statistic_2dp, signif))
      result_output <- bind_rows(result_output, anova_output)
    }
  }
  
  return(result_output)
}

## Run function &  save output
result_output <- compute_aov(dat)
dir.create("output/stat_analysis",showWarnings = TRUE)
write_csv(result_output, paste0("output/stat_analysis/aov_output_raw.csv"))


## Tukey
compute_aov_tukey <- function(x)
{
  dat <- x
  dat[[1]] <- factor(dat[[1]])
  dat[[2]] <- factor(dat[[2]])
  
  for (i in 3:length(colnames(dat)))
  {
    print(colnames(dat)[i])
    # if(i != 31 && i != 32 && i != 44 && i != 45) #drop variables which are not present in both treatments
    {
      summarytable <- tidy(aov(dat[[i]] ~ Geno * Treatment, data = dat, na.action=na.omit))
      write_csv(summarytable, paste0("output/stat_analysis/",colnames(dat)[i],"_aov.csv"))
      
      tempdf <- dat[c(1,2,i)];
      thsd <- tidy(TukeyHSD(aov(tempdf[[3]] ~ Geno * Treatment, data=tempdf, na.action=na.omit)))
      write_csv(thsd, paste0("output/stat_analysis/TukeyHSD/",colnames(dat)[i],"_TukeyHSD.csv"))
    }
  }
}

## Run function & save output
dir.create("output/stat_analysis/TukeyHSD",showWarnings = TRUE)
compute_aov_tukey(dat)

##############
## Plotting ##
##############
filtereddata <- dat_meanErrors %>% filter(Geno=="47" | Geno=="52")

p1 <- ggplot(filtereddata, aes(x=Geno, y=TLA_mm_mean, fill=Treatment), color=factor(Treatment)) +
        geom_bar(position = position_dodge(width=.9), stat="identity") +
        geom_errorbar(aes(ymin=TLA_mm_mean-TLA_mm_se, ymax=TLA_mm_mean+TLA_mm_se), position = position_dodge(width = 0.9), width=.2) +
        theme_bw() +
        theme(
          plot.background = element_blank()
          ,panel.grid.major = element_blank()
          ,panel.grid.minor = element_blank()
        ) + 
  scale_y_continuous(limits=c(0,3000)) +
        xlab(bquote("Genotype")) + 
        ylab(bquote("Total root length (mm" ~ plant^-1*"  SE)"))

p2 <- ggplot(filtereddata, aes(x=Geno, y=TLS_mm_mean, fill=Treatment), color=factor(Treatment)) +
  geom_bar(position = position_dodge(width=.9), stat="identity") +
  geom_errorbar(aes(ymin=TLS_mm_mean-TLS_mm_se, ymax=TLS_mm_mean+TLS_mm_se), position = position_dodge(width = 0.9), width=.2) +
  theme_bw() +
  theme(
    plot.background = element_blank()
    ,panel.grid.major = element_blank()
    ,panel.grid.minor = element_blank()
  ) + 
  scale_y_continuous(limits=c(0,3000)) +
  xlab(bquote("Genotype")) + 
  ylab(bquote("Seminal root length (mm" ~ plant^-1*"  SE)"))

p3 <- ggplot(filtereddata, aes(x=Geno, y=TLL_mm_mean, fill=Treatment), color=factor(Treatment)) +
  geom_bar(position = position_dodge(width=.9), stat="identity") +
  geom_errorbar(aes(ymin=TLL_mm_mean-TLL_mm_se, ymax=TLL_mm_mean+TLL_mm_se), position = position_dodge(width = 0.9), width=.2) +
  theme_bw() +
  theme(
    plot.background = element_blank()
    ,panel.grid.major = element_blank()
    ,panel.grid.minor = element_blank()
  ) + 
  scale_y_continuous(limits=c(0,3000)) +
  xlab(bquote("Genotype")) + 
  ylab(bquote("Lateral root length (mm" ~ plant^-1*"  SE)"))

p4 <- ggplot(filtereddata, aes(x=Geno, y=LC_mean, fill=Treatment), color=factor(Treatment)) +
  geom_bar(position = position_dodge(width=.9), stat="identity") +
  geom_errorbar(aes(ymin=LC_mean-LC_se, ymax=LC_mean+LC_se), position = position_dodge(width = 0.9), width=.2) +
  theme_bw() +
  theme(
    plot.background = element_blank()
    ,panel.grid.major = element_blank()
    ,panel.grid.minor = element_blank()
  ) + 
  xlab(bquote("Genotype")) + 
  ylab(bquote("Lateral root count ( SE)"))

p1 + p2 + p3 + p4 + plot_layout(ncol=4) + 
  plot_layout(guides="collect") +
  plot_annotation(tag_levels = 'A') & theme(plot.tag = element_text(size = 18, face = 'bold'))
ggsave(file=paste("output/Figure4.png"), width=8, height=3.1968, dpi=300)
ggsave(file=paste("output/Figure4.pdf"), width=8, height=3.1968, useDingbats=FALSE)


p4 <- ggplotRegression(dat, "RCH_mm3", "Percent50Cm3") +
  xlab(bquote("Convex hull of root system ("~mm^-3 ~ plant^-1*")")) + 
  ylab(bquote("NUp at final uCT scan (µmol)"))


# p4 <- ggplot(dat_meanErrors, aes(x=Geno, y=TLA_mm_mean, fill=Treatment), color=factor(Treatment)) +
#           geom_bar(position = position_dodge(width=.9), stat="identity") +
#           geom_errorbar(aes(ymin=TLA_mm_mean-TLA_mm_se, ymax=TLA_mm_mean+TLA_mm_se), position = position_dodge(width = 0.9), width=.2) +
#           theme_bw() +
#           theme(
#             plot.background = element_blank()
#             ,panel.grid.major = element_blank()
#             ,panel.grid.minor = element_blank()
#             ,axis.text.x = element_text(angle=45,hjust=1)
#           ) +
#           # scale_y_continuous(limits=c(0,3000)) +
#           xlab(bquote("Genotype")) + 
#           ylab(bquote("Total root length (mm" ~ plant^-1*" ? SE)"))
# 
# p5 <- ggplot(dat_meanErrors, aes(x=Geno, y=TLL_mm_mean, fill=Treatment), color=factor(Treatment)) +
#         geom_bar(position = position_dodge(width=.9), stat="identity") +
#         geom_errorbar(aes(ymin=TLL_mm_mean-TLL_mm_se, ymax=TLL_mm_mean+TLL_mm_se), position = position_dodge(width = 0.9), width=.2) +
#         theme_bw() +
#         theme(
#           plot.background = element_blank()
#           ,panel.grid.major = element_blank()
#           ,panel.grid.minor = element_blank()
#           ,axis.text.x = element_text(angle=45,hjust=1)
#         ) +
#         # scale_y_continuous(limits=c(0,3000)) +
#         xlab(bquote("Genotype")) + 
#         ylab(bquote("Lateral root length (mm" ~ plant^-1*" ? SE)"))
# 
# p6 <- ggplot(dat_meanErrors, aes(x=Geno, y=LC_mean, fill=Treatment), color=factor(Treatment)) +
#         geom_bar(position = position_dodge(width=.9), stat="identity") +
#         geom_errorbar(aes(ymin=LC_mean-LC_se, ymax=LC_mean+LC_se), position = position_dodge(width = 0.9), width=.2) +
#         theme_bw() +
#         theme(
#           plot.background = element_blank()
#           ,panel.grid.major = element_blank()
#           ,panel.grid.minor = element_blank()
#           ,axis.text.x = element_text(angle=45,hjust=1)
#         ) +
#         # scale_y_continuous(limits=c(0,3000)) +
#         xlab(bquote("Genotype")) + 
#         ylab(bquote("Lateral root count (? SE)"))


ggplotRegression  <- function(dat, xvar, yvar){
  fml <- paste(yvar, "~", xvar)
  fit <- lm(fml, dat)
  ggplot(fit$model, aes_string(x = names(fit$model)[2], y = names(fit$model)[1])) + {
    if(signif(summary(fit)$coef[2,4] > 0.05)) stat_smooth(method = "lm", col = "red", linetype = 2) else stat_smooth(method = "lm", col = "red", linetype = 1)
  } +
    geom_point() +
    theme_bw() + 
    theme(
      plot.background = element_blank()
      ,panel.grid.major = element_blank()
      ,panel.grid.minor = element_blank()
    ) +
    geom_text(aes(x=-Inf, y=Inf, hjust=0, vjust=1, label = paste(" R2 = ",signif(summary(fit)$adj.r.squared, 5)
                                                                 ,"Intercept =",signif(fit$coef[[1]],5 )
                                                                 ,"\n"
                                                                 ,"Slope =",signif(fit$coef[[2]], 5)
                                                                 ,"P =",signif(summary(fit)$coef[2,4], 5))))
}

## part 2 plot graph
p7 <- ggplotRegression(dat, "TLL_mm", "TotalNUpDay13") +
  xlab(bquote("Lateral root length (mm" ~ plant^-1*")")) + 
  ylab(bquote("NUp at final uCT scan (µmol)"))

p8 <- ggplotRegression(dat, "LC", "TotalNUpDay13") +
  xlab(bquote("Lateral root count (count" ~ plant^-1*")")) + 
  ylab(bquote("NUp at final uCT scan (µmol)"))

p9 <- ggplotRegression(dat, "RCH_mm3", "TotalNUpDay13") +
  xlab(bquote("Convex hull of root system ("~mm^3 ~ plant^-1*")")) +
  ylab(bquote("NUp at final uCT scan (µmol)"))

p10 <- ggplotRegression(dat, "TLL_mm", "Percent50Cm3") +
  xlab(bquote("Lateral root length (mm" ~ plant^-1*")")) +
  ylab(bquote("Soil zones with 50% nitrate depletion (%)"))

p11 <- ggplotRegression(dat, "LC", "Percent50Cm3") +
  xlab(bquote("Lateral root count (count" ~ plant^-1*")")) +
  ylab(bquote("Soil zones with 50% nitrate depletion (%)"))

p12 <- ggplotRegression(dat, "RCH_mm3", "Percent50Cm3") +
  xlab(bquote("Convex hull of root system ("~mm^3 ~ plant^-1*")")) + 
  ylab(bquote("Soil zones with 50% nitrate depletion (%)"))

p7 + p8 + p9 + p10 + p11 + p12 + plot_layout(ncol=3) +
  plot_layout(guides="collect") +
  plot_annotation(tag_levels = 'A') & theme(plot.tag = element_text(size = 18, face = 'bold'))
ggsave(file=paste("output/Figure5.png"), width=10, height=7, dpi=300)
ggsave(file=paste("output/Figure5.pdf"), width=10, height=7, useDingbats=FALSE)
