from numpy import array
from math import floor
from numpy.linalg import norm
from math import pi,sin,cos,acos
import sys
from numpy import degrees,dot,cross,arctan2

class pyPlant(object):
    def __init__(self):
        self.children={}
        self.parent={}
        self.rootTypes={}
        self.rootPoints={}
        #self.rootDiameters={}
        self.length={}
        self.branchLengths={}
        self.branchPositions={}
        self.axAngle={}
        self.meanWidth={}
        self.pointWidth={}
        self.unit='cm'
        self.resolution=1.0
        self.growthrate={}
        self.metadata={}
        #self.startTimes={}
        self.name='none'

    def setName(self,name):
        self.name=shortname.split('_')[0]
        try:
            self.genotype=name.split('_')[1]
        except IndexError:
            self.genotype='none'
        try:
            self.treatment=name.split('_')[2]
        except IndexError:
            self.treatment='none'
        try:
            self.rep=shortname.split('_')[3]
        except IndexError:
            self.rep='none'

    def typeNames(self):
        return list(set(self.rootTypes.values()))

    def propByType(self,prop):
        return {rtype:[propval for rid,propval in self.prop.items() if self.rootTypes[rid]==rtype] for rtype in self.typeNames()}


    def countRoots(self):
        self.rootCounts={rtype:self.rootTypes.values().count(rtype) for rtype in self.typeNames()}

    def widthsByType(self):
        self.typeWidths=self.propByType(self.meanWidth)
    
    def lengthsByType(self):
        self.typeLengths=self.propByType(self.length)

    def angleByType(self):
        self.typeAngles=self.propByType(self.axAngle)

    def findExactPtInList(self,pt,tpts):
        pref=-1
        for i,xpt in enumerate(tpts):
            if norm(array(xpt)-array(pt))==0.0:
                pref=i
        assert pref>-1
        return pref

    def findNearPtInList(self,pt,tpts):
        pref=-1
        minfound=99999999
        for i,xpt in enumerate(tpts):
            diff=norm(array(xpt)-array(pt))
            if diff<minfound:
                pref=i
                minfound=diff
        assert pref>-1
        return pref

    def pointDistanceFromRef(self,rid,ref,distance):
        pts=self.rootPoints[rid]
        nref=ref+1
        chosen_point=pts[nref]
        while norm(array(pts[nref])-array(pts[ref]))<distance:
                nref+=1
                if nref>=len(pts):
                    break
                chosen_point=pts[nref]
        return chosen_point


    def resetAxAngles(self,distance,absolute=False,distance_multiplier=1.0):
        for rid,pts in self.rootPoints.items():
            if self.rootTypes[rid]!='hypocotyl':
                if absolute==True or self.rootTypes[rid][:8]=='primary':
                    parent_vec=array([0.0,-1.0,0.0])
                else:
                    position=self.findNearPtInList(pts[0],self.rootPoints[self.parent[rid]])
                    ptOnParent=self.pointDistanceFromRef(self.parent[rid],position,distance)
                    parent_vec=array(ptOnParent)-array(pts[0])

                ptOnChild=self.pointDistanceFromRef(rid,0,distance*distance_multiplier)
                child_vec=array(ptOnChild)-array(pts[0])
            
                self.axAngle[rid] = degrees(self.vec_ang(child_vec,parent_vec))
            
                
        
    def vec_ang(self,v1, v2):
        """ Returns the angle in radians between vectors 'v1' and 'v2'    """
        cosang = dot(v1, v2)
        sinang = norm(cross(v1, v2))
        angle = arctan2(sinang, cosang)
        if angle>pi:
            angle=2.0*pi-angle
        return angle

    def radians(self,angle):
        return 2*pi*(angle/360.0)

    def degrees(self,angle):
        return 360.0*(angle/2*pi)


    def getRootPoints(self,start,finish):

        ast=array(start)#start point
        afin=array(finish)#end point
        rlen=norm(afin-ast)#root length
        dvec=(afin-ast)/rlen#unit direction vector
        npts=int(floor(rlen/self.resolution))#no. of extra points needed        

        all_pts=[start]
        for i in range(npts):
            all_pts.append(list(ast+(i+1)*self.resolution*dvec))
        if floor(rlen/self.resolution)!=rlen:
            all_pts.append(finish)
        
        return all_pts

    def getPrimId(self):
        primId=None
        for rid,rtype in self.rootTypes.items():
            if rtype=='primaryRoot' or rtype=='primary':
                primId=rid
        if primId==None:
            print('no primary root defined')
            sys.exit()
        return primId


    def addPrimary(self,primId,axAngle,radAngle,length,width):


        #TODO: make 3D with radAngle
        pDir=[sin(self.radians(axAngle)),0.0,cos(self.radians(axAngle))]

        endPt=list(length*array(pDir))

        self.rootPoints[primId]=self.getRootPoints([0.0,0.0,0.0],endPt)
        self.length[primId]=length
        self.axAngle[primId]=axAngle
        self.meanWidth[primId]=width
        self.rootTypes[primId]='primary'
        self.children[primId]=[]
        self.branchPositions[primId]=[]     
        self.branchLengths[primId]=[]
        self.pointWidth[primId]=[width for pt in self.rootPoints[primId]]   

    def addBranch(self,rid,relBranchPoint,axAngle,radAngle,length,width,parent_id):

        parentPoints=self.rootPoints[parent_id]
        
        #TODO: make 3D with radAngle

        pLength=norm(array(parentPoints[-1])-array(parentPoints[0]))
        pYChange=parentPoints[0][1]-parentPoints[-1][1]
        pAngle=acos(pYChange/pLength)
        bDir=[sin(self.radians(axAngle)+pAngle),-cos(self.radians(axAngle)+pAngle),0.0]


        branchPoint=list(array(parentPoints[0])+relBranchPoint*(array(parentPoints[-1])-array(parentPoints[0])))
        endPt=list(array(branchPoint)+length*array(bDir))
        self.rootPoints[rid]=self.getRootPoints(branchPoint,endPt)
        self.length[rid]=length
        self.axAngle[rid]=axAngle
        self.meanWidth[rid]=width
        self.parent[rid]=parent_id
        self.rootTypes[rid]='lateral'
        self.children[rid]=[]
        self.children[parent_id].append(rid)
        self.branchPositions[parent_id].append(relBranchPoint*self.length[parent_id])
        self.branchPositions[rid]=[]
        self.branchLengths[rid]=[]  
        self.branchLengths[parent_id].append(length)
        self.pointWidth[rid]=[width for pt in self.rootPoints[rid]] 

    def setConnectivity(self,children):
        self.children=children
        self.parent={}

        for pid,kids in children.items():
            for kid in kids:
                if kid in self.parent:
                    print('non-unique parent for root',kid)
                else:
                    self.parent[kid]=pid

        
            
        
        
        
        
        

